###################################################################
# identify_rocks.py
# or "07 Identify Rock Outcrops" tool in the SICCM toolbox
#
# By Michael Bunn, Ben Leshchinsky, and Michael Olsen
# Oregon State University
# Last Updated: 07/12/2018
#
# Creation of this tool has been supported by Oregon Department of
# Transportation research project SPR 786 and
# United States Forest Service grant number 17-CS-11015600-008
#
# Direct questions to ben.leshchinsky@oregonstate.edu
###################################################################

# Import arcpy module
import arcpy, os
from arcpy.sa import *
arcpy.env.overwriteOutput=True

# Check out any necessary licenses
arcpy.CheckOutExtension("spatial")

# Get the input
input_raster = arcpy.GetParameterAsText(0)

# Find location of input
descInput = arcpy.Describe(input_raster)
arcpy.env.workspace = os.path.dirname(descInput.catalogPath)

# Display output location
loc = arcpy.env.workspace
commentval = "Outputs will be saved to "+loc
arcpy.AddMessage(commentval)

# Determine cellsize and unit of input DEM
description = arcpy.Describe(input_raster)
cellsize = description.children[0].meanCellHeight

spat = arcpy.Describe(input_raster).spatialReference
unit = spat.linearUnitName
arcpy.AddMessage(unit)
# List temporary variables
resampled_raster = "resampled_raster"
output_slope = "output_slope"
output_profile = "output_profile"
output_curva = "output_curva"
output_plan = "output_plan"
reclass_slope = "reclass_slope"
reclass_profile = "reclass_profile"
reclass_times = "reclass_times"
reclass2_times = "reclass2_times"
pct_reclass = "pct_reclass"
rock_raster = "rock_raster"
all_polygons = "all_polygons"
rock_polygons = "rock_polygons"

if unit == "Foot" or unit == "FootUS":
	Output_Cell_Size = "30 30"
	if cellsize > 30:
		# Resample DEM to 30 foot cell size

		# Tell the user that results will not be optimal
		arcpy.AddWarning('Since the input DEM has a cell size > 30 feet, results will not be optimal')
elif unit == "Meter":
	Output_Cell_Size = "10 10"
	if cellsize > 10:
		# Resample DEM to 10 meter cell size
		

		# Tell the user that results will not be optimal
		arcpy.AddWarning('Since the input DEM has a cell size > 10 meters, results will not be optimal')

# Resample DEM
arcpy.Resample_management(input_raster, resampled_raster, Output_Cell_Size, "BILINEAR")

# Compute slope and profile curvature from resampled DEM
outSlope = Slope(resampled_raster, "DEGREE", "1")
outSlope.save(output_slope)

outCurve = Curvature(resampled_raster, "1", output_profile, output_plan)
outCurve.save(output_curva)

# Reclassify slope and profile curvature
remap_slope = RemapRange([[0, 36, 0], [36, 54, 1], [54, 100, 1]])
remap_profile = RemapRange([[-1000000, -3.5, 1], [-3.5, 2.25, 0], [2.25, 1000000, 1]])

SlopeReclass = Reclassify(output_slope, "VALUE", remap_slope, "DATA")
SlopeReclass.save(reclass_slope)
ProfileReclass = Reclassify(output_profile, "VALUE", remap_profile, "DATA")
ProfileReclass.save(reclass_profile)

# Multiply the reclassified layers together
outTimes = Times(reclass_slope, reclass_profile)
outTimes.save(reclass_times)

# Fail safe reclass_profile
remap_fail = RemapRange([[0, 0.5, 0], [0.5, 1000000, 1]])
FailReclass = Reclassify(reclass_times,"VALUE", remap_fail, "DATA")
FailReclass.save(reclass2_times)

# Perform focal statistics
if unit == "Foot" or unit == "FootUS":
	neigh = NbrCircle( 82, "MAP")
elif unit == "Meter":
	neigh = NbrCircle( 25, "MAP")
	
OutFocal = FocalStatistics(reclass2_times, neigh, "MEAN", "DATA")
OutFocal.save(pct_reclass)

# Reclassify the focal statistics output
remap_pct = RemapRange([[0, 0.16, 0], [0.16, 1, 1]])
PctReclass = Reclassify(pct_reclass, "VALUE", remap_pct, "DATA")
PctReclass.save(rock_raster)

# Convert raster into polygons
arcpy.RasterToPolygon_conversion(rock_raster, all_polygons, "SIMPLIFY", "VALUE")
arcpy.MakeFeatureLayer_management(all_polygons, "polys")
arcpy.SelectLayerByAttribute_management ("polys", "NEW_SELECTION", '"GRIDCODE" =1')
arcpy.CopyFeatures_management("polys", rock_polygons)
arcpy.Delete_management("polys")

# Delete unnecessary files
arcpy.Delete_management(resampled_raster)
arcpy.Delete_management(output_slope)
arcpy.Delete_management(output_profile)
arcpy.Delete_management(output_curva)
arcpy.Delete_management(output_plan)
arcpy.Delete_management(reclass_slope)
arcpy.Delete_management(reclass_profile)
arcpy.Delete_management(reclass_times)
arcpy.Delete_management(reclass2_times)
arcpy.Delete_management(pct_reclass)
arcpy.Delete_management(rock_raster)
arcpy.Delete_management(all_polygons)

## Display Rock Outcrops on map

# Find location of layer file
lyrDir = os.path.dirname(__file__)

fileRock = lyrDir+"\\rockoutcrops.lyr"

# Set the map document and data frame
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"Layers")[0]

# Create layer file and add to map
lyrRock = arcpy.mapping.Layer(rock_polygons)
arcpy.ApplySymbologyFromLayer_management(lyrRock, fileRock)
arcpy.mapping.AddLayer(df,lyrRock)

arcpy.CheckInExtension("spatial")