#############################################################
# run_ccm.py
# or "11 Run CCM" tool in the CCM Flow toolbox
#
# By Michael Bunn, Ben Leshchinsky, and Michael Olsen
# Oregon State University
# Last Updated: 02/14/2018
#
# Creation of this tool has been supported by Oregon Department of
# Transportation research project SPR 786 and
# United States Forest Service grant number 17-CS-11015600-008
#
# Direct questions to ben.leshchinsky@oregonstate.edu
#############################################################

import arcpy, math, os, shutil, subprocess

# Define conversion function##########################################
def LinearUnit2Pixel(linear_unit,input_raster):
	# Determine the area of an individual DEM cell
	# Units are in raster_units squared
	description = arcpy.Describe(input_raster)
	cell_size = description.children[0].meanCellHeight
	cell_area = cell_size*cell_size
	
	spat = arcpy.Describe(input_raster).spatialReference
	raster_units = spat.linearUnitName
	
	# Create a dictionary with conversion values
	conversions = {"Feet":[1,0.3048], \
	"Centimeters":[0.0328,0.01], \
	"Decimeters":[0.3281,0.1], \
	"Inches":[0.0833,0.0254], \
	"Kilometers":[3280.84,1000], \
	"Meters":[3.2808,1], \
	"Miles":[5280,1609], \
	"Millimeters":[0.0033,0.001], \
	"NauticalMiles":[6076,1852], \
	"Yards":[3,0.9144]}
	
	# Look up conversion factors
	try:
		if raster_units == "Meter":
			# Input raster in meters
			conv_factor = conversions[linear_unit][1]
		elif raster_units == "Foot":
			# Input raster in feet
			conv_factor = conversions[linear_unit][0]
		elif raster_units == "Foot_US":
			# Input raster in feet
			conv_factor = conversions[linear_unit][0]
	except:
		arcpy.AddError("Selected unit unavailable. Please input another unit.")
		conv_factor = 1
	
	try:
		return conv_factor
	except:
		arcpy.AddError("'conv_factor' is empty. Please check that you have specified Contour Interval and Node Spacing units, and that your DEM has a spatial reference") 
######################################################################




## REQUIRED INPUTS
# Information from a CCM Package
CCM_Package = arcpy.GetParameterAsText(0)
DEM_FP = CCM_Package+"\\CCM_DEM.tif"
SCARP_FP = CCM_Package+"\\CCM_scarps.shp"
OUTPUT_FP = CCM_Package

# Create CCM Run File Geodatabase
RepDir, PackName = os.path.split(CCM_Package)
newGDBdir = CCM_Package+"\\"+PackName+".gdb"
if not os.path.exists(newGDBdir):
	arcpy.CreateFileGDB_management(CCM_Package, PackName, "10.0")

arcpy.env.workspace = newGDBdir
save_message = "Created new File GDB for CCM Package: "+newGDBdir
arcpy.AddMessage(save_message)

# Other inputs
contour_interval = arcpy.GetParameterAsText(1)
ici = contour_interval.find(" ")
ici1 = int(ici+1)
unit_ci = contour_interval[ici1:]
conv_ci = LinearUnit2Pixel(unit_ci,DEM_FP)

CONTOUR = int(round((int(contour_interval[:ici])*conv_ci)))

node_spacing = arcpy.GetParameterAsText(2)
ins = node_spacing.find(" ")
ins1 = int(ins+1)
unit_ns = node_spacing[ins1:]
conv_ns = LinearUnit2Pixel(unit_ns,DEM_FP)

SPACING = int(round((int(node_spacing[:ins])*conv_ns)))

active_slope = arcpy.GetParameter(3)
SLOPE = round(math.tan(math.radians(active_slope)),3)
BRANCHES = arcpy.GetParameter(4)

## OPTIONAL INPUTS - ACCORDING TO CCM FLOW
if unit_ci == "Meter":
	contour = str(CONTOUR)+"m"
else:
	contour = str(CONTOUR)+"ft"
if unit_ns == "Meter":
	nodal = str(SPACING)+"m"
else:
	nodal = str(SPACING)+"ft"
	
run_name = "CI"+contour+"_NS"+nodal+"_AS"+str(int(active_slope))+"deg"+"_BP"+str(BRANCHES)
model_name = " --model_name "+run_name
id_field = " --id_field Id"


# Fixed - will never change
metadata = " --no-metadata"
composite_metadata = " --no-composite-metadata"
composite_statistics = " --composite-statistics"
composite_nodes = " --no-composite-nodes"
composite_connections = " --no-composite-connections"
composite_extent = " --composite-extent"

# Optional - user can choose all or none
individual_files = arcpy.GetParameterAsText(7)
if individual_files:
	individual_statistics = " --individual-statistics"
	individual_nodes = " --individual-nodes"
	individual_connections = " --individual-connections"
	individual_extent = " --individual-extent"
else:
	individual_statistics = " --no-individual-statistics"
	individual_nodes = " --no-individual-nodes"
	individual_connections = " --no-individual-connections"
	individual_extent = " --no-individual-extent"

# Optional - user decides whether or not to remove tails
remove_tails = " --remove-tails"
tails_cutoff = arcpy.GetParameter(5)
if tails_cutoff == '#' or not tails_cutoff:
	remove_tails = " --no-remove-tails"
	tails_threshold = ""
	tails_remove_after = ""
else:
	remove_tails = " --remove-tails"
	tails_threshold = " --tails-threshold "+str(round((tails_cutoff/float(SPACING)),3))
	tail_remove = arcpy.GetParameterAsText(6)
	if tail_remove =='#' or not tail_remove:
		tails_remove_after = " --tails-remove-after-layer "+"3"
	else:
		tails_remove_after = " --tails-remove-after-layer "+arcpy.GetParameterAsText(6)




ccm_input = "ccmflow-x64-cli.exe "+model_name+id_field+ \
	metadata + composite_metadata + composite_statistics + \
	composite_nodes + composite_connections + composite_extent + \
	individual_statistics + individual_nodes + \
	individual_connections + individual_extent + remove_tails + \
	tails_threshold + tails_remove_after + \
	" " + DEM_FP + " " + SCARP_FP + " " + OUTPUT_FP + " " + \
	str(CONTOUR) + " " + str(SPACING) + " " + str(SLOPE) + " " + \
	str(BRANCHES)

ccmdir = OUTPUT_FP+"\\"+run_name
extent_name = ccmdir+"\\"+run_name+"_extent_poly.shp"
if os.path.exists(ccmdir):
	shutil.rmtree(ccmdir)

os.chdir('C:\Program Files\CCMFlow-x64-cli')
subprocess.check_call(ccm_input, stdin=None, stdout=None, stderr=None, shell=False)



#try:
arcpy.CopyFeatures_management(extent_name,run_name)
#except:
#arcpy.AddMessage("Failure during CCM run or extent poly has name with decimals")
arcpy.RefreshCatalog(CCM_Package)
## Display extents on map

# Set the map document and data frame
mxd = arcpy.mapping.MapDocument("CURRENT")
df = arcpy.mapping.ListDataFrames(mxd,"Layers")[0]


# Create layer file and add to map
lyrDir = os.path.dirname(__file__)

fileExt = lyrDir+"\\extents.lyr"
lyrExt = arcpy.mapping.Layer(run_name)
arcpy.ApplySymbologyFromLayer_management(lyrExt, fileExt)
arcpy.mapping.AddLayer(df,lyrExt)